@@ -1,12 +1,10 @@
-# © 2017 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
-
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
 """Interface to Ghostscript executable"""
 
+from __future__ import annotations
+
 import logging
 import os
 import re
@@ -14,41 +12,35 @@ import sys
 from io import BytesIO
 from os import fspath
 from pathlib import Path
-from shutil import which
 from subprocess import PIPE, CalledProcessError
-from typing import Optional
 
 from PIL import Image, UnidentifiedImageError
 
-from ocrmypdf.exceptions import MissingDependencyError, SubprocessOutputError
+from ocrmypdf.exceptions import SubprocessOutputError
 from ocrmypdf.helpers import Resolution
 from ocrmypdf.subprocess import get_version, run, run_polling_stderr
 
+# Remove this workaround when we require Pillow >= 10
+try:
+    Transpose = Image.Transpose  # type: ignore
+except AttributeError:
+    # Pillow 9 shim
+    Transpose = Image  # type: ignore
+
 log = logging.getLogger(__name__)
 
-missing_gs_error = """
----------------------------------------------------------------------
-This error normally occurs when ocrmypdf find can't Ghostscript.
-Please ensure Ghostscript is installed and its location is added to
-the system PATH environment variable.
-
-For details see:
-    https://ocrmypdf.readthedocs.io/en/latest/installation.html
----------------------------------------------------------------------
-"""
-
 # Most reliable what to get the bitness of Python interpreter, according to Python docs
-_is_64bit = sys.maxsize > 2 ** 32
+_IS_64BIT = sys.maxsize > 2**32
 
-_gswin = None
+_GSWIN = None
 if os.name == 'nt':
-    if _is_64bit:
-        _gswin = 'gswin64c'
+    if _IS_64BIT:
+        _GSWIN = 'gswin64c'
     else:
-        _gswin = 'gswin32c'
+        _GSWIN = 'gswin32c'
 
-GS = _gswin if _gswin else 'gs'
-del _gswin
+GS = _GSWIN if _GSWIN else 'gs'
+del _GSWIN
 
 
 def version():
@@ -82,8 +74,8 @@ def rasterize_pdf(
     raster_device: str,
     raster_dpi: Resolution,
     pageno: int = 1,
-    page_dpi: Optional[Resolution] = None,
-    rotation: Optional[int] = None,
+    page_dpi: Resolution | None = None,
+    rotation: int | None = None,
     filter_vector: bool = False,
 ):
     """Rasterize one page of a PDF at resolution raster_dpi in canvas units."""
@@ -108,7 +100,7 @@ def rasterize_pdf(
         + [
             '-o',
             '-',
-            '-sstdout=%stderr',
+            '-sstdout=%stderr',  # Literal %s, not string interpolation
             '-dAutoRotatePages=/None',  # Probably has no effect on raster
             '-f',
             fspath(input_file),
@@ -119,7 +111,7 @@ def rasterize_pdf(
         p = run(args_gs, stdout=PIPE, stderr=PIPE, check=True)
     except CalledProcessError as e:
         log.error(e.stderr.decode(errors='replace'))
-        raise SubprocessOutputError('Ghostscript rasterizing failed')
+        raise SubprocessOutputError('Ghostscript rasterizing failed') from e
     else:
         stderr = p.stderr.decode(errors='replace')
         if _gs_error_reported(stderr):
@@ -132,11 +124,11 @@ def rasterize_pdf(
                 # rotation is a clockwise angle and Image.ROTATE_* is
                 # counterclockwise so this cancels out the rotation
                 if rotation == 90:
-                    im = im.transpose(Image.ROTATE_90)
+                    im = im.transpose(Transpose.ROTATE_90)
                 elif rotation == 180:
-                    im = im.transpose(Image.ROTATE_180)
+                    im = im.transpose(Transpose.ROTATE_180)
                 elif rotation == 270:
-                    im = im.transpose(Image.ROTATE_270)
+                    im = im.transpose(Transpose.ROTATE_270)
                 if rotation % 180 == 90:
                     page_dpi = page_dpi.flip_axis()
             im.save(fspath(output_file), dpi=page_dpi)
@@ -149,6 +141,8 @@ def rasterize_pdf(
 
 
 class GhostscriptFollower:
+    """Parses the output of Ghostscript and uses it to update the progress bar."""
+
     re_process = re.compile(r"Processing pages \d+ through (\d+).")
     re_page = re.compile(r"Page (\d+)")
 
@@ -210,14 +204,18 @@ def generate_pdfa(
     # Older versions of Ghostscript expect a leading slash in
     # sColorConversionStrategy, newer ones should not have it. See Ghostscript
     # git commit fe1c025d.
-    strategy = ('/' + strategy) if version() < '9.19' else strategy
+    gs_version = version()
+    strategy = ('/' + strategy) if gs_version < '9.19' else strategy
 
-    if version() == '9.23':
+    if gs_version == '9.23':
         # 9.23: added JPEG passthrough as a new feature, but with a bug that
         # incorrectly formats some images. Fixed as of 9.24. So we disable this
         # feature for 9.23.
         # https://bugs.ghostscript.com/show_bug.cgi?id=699216
         compression_args.append('-dPassThroughJPEGImages=false')
+    elif gs_version == '9.56.0':
+        # 9.56.0 breaks our OCR...?
+        compression_args.append('-dNEWPDF=false')
 
     # nb no need to specify ProcessColorModel when ColorConversionStrategy
     # is set; see:
@@ -240,7 +238,7 @@ def generate_pdfa(
             "-dPDFACompatibilityPolicy=1",
             "-o",
             "-",
-            "-sstdout=%stderr",
+            "-sstdout=%stderr",  # Literal %s, not string interpolation
         ]
     )
     args_gs.extend(fspath(s) for s in pdf_pages)  # Stringify Path objs
