@@ -1,19 +1,16 @@
-# © 2017 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
-
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
 """Interface to Tesseract executable"""
 
+from __future__ import annotations
+
 import logging
 import re
 from math import pi
 from os import fspath
 from pathlib import Path
 from subprocess import PIPE, STDOUT, CalledProcessError, TimeoutExpired
-from typing import Dict, Iterator, List, Optional
 
 from packaging.version import Version
 from PIL import Image
@@ -46,7 +43,7 @@ HOCR_TEMPLATE = """<?xml version="1.0" encoding="UTF-8"?>
 </html>
 """
 
-TESSERACT_THRESHOLDING_METHODS: Dict[str, int] = {
+TESSERACT_THRESHOLDING_METHODS: dict[str, int] = {
     'auto': 0,
     'otsu': 0,
     'adaptive-otsu': 1,
@@ -55,9 +52,11 @@ TESSERACT_THRESHOLDING_METHODS: Dict[str, int] = {
 
 
 class TesseractLoggerAdapter(logging.LoggerAdapter):
+    "Prepend [tesseract] to messages emitted from tesseract"
+
     def process(self, msg, kwargs):
         kwargs['extra'] = self.extra
-        return '[tesseract] %s' % (msg), kwargs
+        return f'[tesseract] {msg}', kwargs
 
 
 TESSERACT_VERSION_PATTERN = r"""
@@ -105,6 +104,7 @@ TESSERACT_VERSION_PATTERN = r"""
 
 
 class TesseractVersion(Version):
+    "Modify standard packaging.Version regex to support Tesseract idiosyncracies."
     _regex = re.compile(
         r"^\s*" + TESSERACT_VERSION_PATTERN + r"\s*$", re.VERBOSE | re.IGNORECASE
     )
@@ -114,7 +114,7 @@ def version() -> str:
     return get_version('tesseract', regex=r'tesseract\s(.+)')
 
 
-def has_user_words():
+def has_user_words() -> bool:
     """Does Tesseract have --user-words capability?
 
     Not available in 4.0, but available in 4.1. Also available in 3.x, but
@@ -123,12 +123,12 @@ def has_user_words():
     return version() >= '4.1'
 
 
-def has_thresholding():
+def has_thresholding() -> bool:
     """Does Tesseract have -c thresholding method capability?"""
     return version() >= '5.0'
 
 
-def get_languages():
+def get_languages() -> set[str]:
     def lang_error(output):
         msg = (
             "Tesseract failed to report available languages.\n"
@@ -159,7 +159,7 @@ def get_languages():
     return {lang.strip() for lang in rest}
 
 
-def tess_base_args(langs: List[str], engine_mode: Optional[int]) -> List[str]:
+def tess_base_args(langs: list[str], engine_mode: int | None) -> list[str]:
     args = ['tesseract']
     if langs:
         args.extend(['-l', '+'.join(langs)])
@@ -168,19 +168,19 @@ def tess_base_args(langs: List[str], engine_mode: Optional[int]) -> List[str]:
     return args
 
 
-def _parse_tesseract_output(binary_output: bytes) -> Dict[str, str]:
-    def g():
+def _parse_tesseract_output(binary_output: bytes) -> dict[str, str]:
+    def gen():
         for line in binary_output.decode().splitlines():
             line = line.strip()
             parts = line.split(':', maxsplit=2)
             if len(parts) == 2:
                 yield parts[0].strip(), parts[1].strip()
 
-    return {k: v for k, v in g()}
+    return dict(gen())
 
 
 def get_orientation(
-    input_file: Path, engine_mode: Optional[int], timeout: float
+    input_file: Path, engine_mode: int | None, timeout: float
 ) -> OrientationConfidence:
     args_tesseract = tess_base_args(['osd'], engine_mode) + [
         '--psm',
@@ -205,14 +205,14 @@ def get_orientation(
 
     osd = _parse_tesseract_output(p.stdout)
     angle = int(osd.get('Orientation in degrees', 0))
-    oc = OrientationConfidence(
+    orient_conf = OrientationConfidence(
         angle=angle, confidence=float(osd.get('Orientation confidence', 0))
     )
-    return oc
+    return orient_conf
 
 
 def get_deskew(
-    input_file: Path, languages: List[str], engine_mode: Optional[int], timeout: float
+    input_file: Path, languages: list[str], engine_mode: int | None, timeout: float
 ) -> float:
     """Gets angle to deskew this page, in degrees."""
     args_tesseract = tess_base_args(languages, engine_mode) + [
@@ -242,7 +242,7 @@ def get_deskew(
     return deskew_degrees
 
 
-def tesseract_log_output(stream):
+def tesseract_log_output(stream: bytes) -> None:
     tlog = TesseractLoggerAdapter(
         log, extra=log.extra if hasattr(log, 'extra') else None
     )
@@ -282,13 +282,13 @@ def tesseract_log_output(stream):
             tlog.info(line.strip())
 
 
-def page_timedout(timeout):
+def page_timedout(timeout: float) -> None:
     if timeout == 0:
         return
     log.warning("[tesseract] took too long to OCR - skipping")
 
 
-def _generate_null_hocr(output_hocr, output_text, image):
+def _generate_null_hocr(output_hocr: Path, output_text: Path, image: Path) -> None:
     """Produce a .hocr file that reports no text detected on a page that is
     the same size as the input image."""
     with Image.open(image) as im:
@@ -303,15 +303,15 @@ def generate_hocr(
     input_file: Path,
     output_hocr: Path,
     output_text: Path,
-    languages: List[str],
+    languages: list[str],
     engine_mode: int,
-    tessconfig: List[str],
+    tessconfig: list[str],
     timeout: float,
     pagesegmode: int,
     thresholding: int,
     user_words,
     user_patterns,
-):
+) -> None:
     """Generate a hOCR file, which must be converted to PDF."""
     prefix = output_hocr.with_suffix('')
 
@@ -357,7 +357,7 @@ def generate_hocr(
             prefix.with_suffix('.txt').replace(output_text)
 
 
-def use_skip_page(output_pdf, output_text):
+def use_skip_page(output_pdf: Path, output_text: Path) -> None:
     output_text.write_text('[skipped page]', encoding='utf-8')
 
     # A 0 byte file to the output to indicate a skip
@@ -369,15 +369,15 @@ def generate_pdf(
     input_file: Path,
     output_pdf: Path,
     output_text: Path,
-    languages: List[str],
+    languages: list[str],
     engine_mode: int,
-    tessconfig: List[str],
+    tessconfig: list[str],
     timeout: float,
     pagesegmode: int,
     thresholding: int,
     user_words,
     user_patterns,
-):
+) -> None:
     """Generate a PDF using Tesseract's internal PDF generator.
 
     We specifically a text-only PDF which is more suitable for combining with
