@@ -1,30 +1,48 @@
-# © 2015 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 # unpaper documentation:
 # https://github.com/Flameeyes/unpaper/blob/master/doc/basic-concepts.md
-
 """Interface to unpaper executable"""
 
 import logging
 import os
 import shlex
+import sys
 from contextlib import contextmanager
 from decimal import Decimal
 from pathlib import Path
 from subprocess import PIPE, STDOUT
-from tempfile import TemporaryDirectory
-from typing import List, Optional, Tuple, Union
+from typing import Iterator, Union
 
 from PIL import Image
 
 from ocrmypdf.exceptions import MissingDependencyError, SubprocessOutputError
 from ocrmypdf.subprocess import get_version, run
 
+if sys.version_info >= (3, 10):
+    from tempfile import TemporaryDirectory
+else:
+    from tempfile import TemporaryDirectory as _TemporaryDirectory
+
+    class TemporaryDirectory(_TemporaryDirectory):
+        """Shim to consume ignore_cleanup_errors kwarg on Python 3.9 and older.
+
+        The argument is consumed without action. If users are getting errors related
+        to temporary file cleanup, they should upgrade to Python 3.10 which properly
+        cleans up temporary directories on Windows.
+
+        See: https://github.com/python/cpython/pull/24793
+        """
+
+        def __init__(self, ignore_cleanup_errors=False, **kwargs):
+            super().__init__(**kwargs)
+
+    del _TemporaryDirectory
+
+
 UNPAPER_IMAGE_PIXEL_LIMIT = 256 * 1024 * 1024
 
 DecFloat = Union[Decimal, float]
@@ -33,6 +51,8 @@ log = logging.getLogger(__name__)
 
 
 class UnpaperImageTooLargeError(Exception):
+    """To capture details when an image is too large for unpaper."""
+
     def __init__(
         self,
         w,
@@ -49,11 +69,13 @@ def version() -> str:
     return get_version('unpaper')
 
 
-def _convert_image(im: Image.Image) -> Tuple[Image.Image, bool, str]:
-    SUFFIXES = {'1': '.pbm', 'L': '.pgm', 'RGB': '.ppm'}
+SUPPORTED_MODES = {'1', 'L', 'RGB'}
+
+
+def _convert_image(im: Image.Image) -> tuple[Image.Image, bool]:
     im_modified = False
 
-    if im.mode not in SUFFIXES:
+    if im.mode not in SUPPORTED_MODES:
         log.info("Converting image to other colorspace")
         try:
             if im.mode == 'P' and len(im.getcolors()) == 2:
@@ -66,41 +88,41 @@ def _convert_image(im: Image.Image) -> Tuple[Image.Image, bool, str]:
             ) from e
         else:
             im_modified = True
-    try:
-        suffix = SUFFIXES[im.mode]
-    except KeyError:
-        raise MissingDependencyError(
-            "Failed to convert image to a supported format."
-        ) from None
-    return im, im_modified, suffix
+        if im.mode not in SUPPORTED_MODES:
+            raise MissingDependencyError(
+                "Failed to convert image to a supported format."
+            ) from None
+    return im, im_modified
 
 
 @contextmanager
-def _setup_unpaper_io(input_file: Path) -> Tuple[Path, Path, Path]:
+def _setup_unpaper_io(input_file: Path) -> Iterator[tuple[Path, Path, Path]]:
     with Image.open(input_file) as im:
         if im.width * im.height >= UNPAPER_IMAGE_PIXEL_LIMIT:
             raise UnpaperImageTooLargeError(w=im.width, h=im.height)
-        im, im_modified, suffix = _convert_image(im)
+        im, im_modified = _convert_image(im)
 
-        with TemporaryDirectory() as tmpdir:
+        with TemporaryDirectory(ignore_cleanup_errors=True) as tmpdir:
             tmppath = Path(tmpdir)
-            if im_modified or input_file.suffix != '.pnm':
-                input_pnm = tmppath / 'input.pnm'
-                im.save(input_pnm, format='PPM')
+            if im_modified or input_file.suffix != '.png':
+                input_png = tmppath / 'input.png'
+                im.save(input_png, format='PNG')
             else:
                 # No changes, PNG input, just use the file we already have
-                input_pnm = input_file
+                input_png = input_file
 
-            output_pnm = tmppath / f'output{suffix}'
-            yield input_pnm, output_pnm, tmppath
+            # unpaper can write .png too, but it seems to write them slowly
+            # adds a few seconds to test suite - so just use pnm
+            output_pnm = tmppath / 'output.pnm'
+            yield input_png, output_pnm, tmppath
 
 
 def run_unpaper(
-    input_file: Path, output_file: Path, *, dpi: DecFloat, mode_args: List[str]
+    input_file: Path, output_file: Path, *, dpi: DecFloat, mode_args: list[str]
 ) -> None:
     args_unpaper = ['unpaper', '-v', '--dpi', str(round(dpi, 6))] + mode_args
 
-    with _setup_unpaper_io(input_file) as (input_pnm, output_pnm, tmpdir):
+    with _setup_unpaper_io(input_file) as (input_png, output_pnm, tmpdir):
         # To prevent any shenanigans from accepting arbitrary parameters in
         # --unpaper-args, we:
         # 1) run with cwd set to a tmpdir with only unpaper's files
@@ -108,7 +130,7 @@ def run_unpaper(
         # 3) append absolute paths for the input and output file
         # This should ensure that a user cannot clobber some other file with
         # their unpaper arguments (whether intentionally or otherwise)
-        args_unpaper.extend([os.fspath(input_pnm), os.fspath(output_pnm)])
+        args_unpaper.extend([os.fspath(input_png), os.fspath(output_pnm)])
         run(
             args_unpaper,
             close_fds=True,
@@ -129,7 +151,7 @@ def run_unpaper(
             ) from e
 
 
-def validate_custom_args(args: str) -> List[str]:
+def validate_custom_args(args: str) -> list[str]:
     unpaper_args = shlex.split(args)
     if any(('/' in arg or arg == '.' or arg == '..') for arg in unpaper_args):
         raise ValueError('No filenames allowed in --unpaper-args')
@@ -141,7 +163,7 @@ def clean(
     output_file: Path,
     *,
     dpi: DecFloat,
-    unpaper_args: Optional[List[str]] = None,
+    unpaper_args: list[str] | None = None,
 ) -> Path:
     default_args = [
         '--layout',
