@@ -1,9 +1,9 @@
-# © 2016 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""OCRmyPDF page processing pipeline functions."""
+
+from __future__ import annotations
 
 import logging
 import os
@@ -13,12 +13,12 @@ from contextlib import suppress
 from datetime import datetime, timezone
 from pathlib import Path
 from shutil import copyfileobj
-from typing import Dict, Iterable, Optional
+from typing import Iterable
 
 import img2pdf
 import pikepdf
 from pikepdf.models.metadata import encode_pdf_date
-from PIL import Image, ImageDraw
+from PIL import Image, ImageColor, ImageDraw
 
 from ocrmypdf._concurrent import Executor
 from ocrmypdf._exec import unpaper
@@ -34,10 +34,16 @@ from ocrmypdf.exceptions import (
 )
 from ocrmypdf.helpers import IMG2PDF_KWARGS, Resolution, safe_symlink
 from ocrmypdf.hocrtransform import HocrTransform
-from ocrmypdf.optimize import optimize
 from ocrmypdf.pdfa import generate_pdfa_ps
 from ocrmypdf.pdfinfo import Colorspace, Encoding, PdfInfo
 
+# Remove this workaround when we require Pillow >= 10
+try:
+    BICUBIC = Image.Resampling.BICUBIC  # type: ignore
+except AttributeError:  # pragma: no cover
+    # Pillow 9 shim
+    BICUBIC = Image.BICUBIC  # type: ignore
+
 log = logging.getLogger(__name__)
 
 VECTOR_PAGE_DPI = 400
@@ -325,7 +331,8 @@ def is_ocr_required(page_context: PageContext):
             ocr_required = False
             log.warning(
                 "page too big, skipping OCR "
-                f"({(pixel_count / 1_000_000):.1f} MPixels > {options.skip_big:.1f} MPixels --skip-big)"
+                f"({(pixel_count / 1_000_000):.1f} MPixels > "
+                f"{options.skip_big:.1f} MPixels --skip-big)"
             )
     return ocr_required
 
@@ -423,8 +430,8 @@ def rasterize(
     output_file = page_context.get_path(f'rasterize{output_tag}.png')
     pageinfo = page_context.pageinfo
 
-    def at_least(cs):
-        return max(device_idx, colorspaces.index(cs))
+    def at_least(colorspace):
+        return max(device_idx, colorspaces.index(colorspace))
 
     for image in pageinfo.images:
         if image.type_ != 'image':
@@ -464,10 +471,10 @@ def rasterize(
 
 def preprocess_remove_background(input_file: Path, page_context: PageContext):
     if any(image.bpc > 1 for image in page_context.pageinfo.images):
-        output_file = page_context.get_path('pp_rm_bg.png')
-        # leptonica.remove_background(input_file, output_file)
         raise NotImplementedError("--remove-background is temporarily not implemented")
-        return output_file
+        # output_file = page_context.get_path('pp_rm_bg.png')
+        # leptonica.remove_background(input_file, output_file)
+        # return output_file
     else:
         log.info("background removal skipped on mono page")
         return input_file
@@ -484,7 +491,9 @@ def preprocess_deskew(input_file: Path, page_context: PageContext):
         # According to Pillow docs, .rotate() will automatically use Image.NEAREST
         # resampling if image is mode '1' or 'P'
         deskewed = im.rotate(
-            deskew_angle_degrees, resample=Image.BICUBIC, fillcolor='white'
+            deskew_angle_degrees,
+            resample=BICUBIC,
+            fillcolor=ImageColor.getcolor('white', mode=im.mode),  # type: ignore
         )
         deskewed.save(output_file, dpi=dpi)
 
@@ -655,7 +664,7 @@ def ocr_engine_textonly_pdf(input_image: Path, page_context: PageContext):
     return (output_pdf, output_text)
 
 
-def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> Dict[str, str]:
+def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> dict[str, str]:
     options = context.options
 
     def from_document_info(key):
@@ -669,15 +678,14 @@ def get_docinfo(base_pdf: pikepdf.Pdf, context: PdfContext) -> Dict[str, str]:
         k: from_document_info(k)
         for k in ('/Title', '/Author', '/Keywords', '/Subject', '/CreationDate')
     }
-    if options is not None:
-        if options.title:
-            pdfmark['/Title'] = options.title
-        if options.author:
-            pdfmark['/Author'] = options.author
-        if options.keywords:
-            pdfmark['/Keywords'] = options.keywords
-        if options.subject:
-            pdfmark['/Subject'] = options.subject
+    if options.title:
+        pdfmark['/Title'] = options.title
+    if options.author:
+        pdfmark['/Author'] = options.author
+    if options.keywords:
+        pdfmark['/Keywords'] = options.keywords
+    if options.subject:
+        pdfmark['/Subject'] = options.subject
 
     creator_tag = context.plugin_manager.hook.get_ocr_engine().creator_tag(options)
 
@@ -807,13 +815,14 @@ def metadata_fixup(working_file: Path, context: PdfContext):
                 missing = set(meta_original.keys()) - set(meta.keys())
                 report_on_metadata(missing)
 
+        optimizing = context.plugin_manager.hook.is_optimization_enabled(
+            context=context
+        )
         pdf.save(
             output_file,
             **get_pdf_save_settings(options.output_type),
             linearize=(  # Don't linearize if optimize() will be linearizing too
-                should_linearize(working_file, context)
-                if options.optimize == 0
-                else False
+                not optimizing and should_linearize(working_file, context)
             ),
         )
 
@@ -822,12 +831,22 @@ def metadata_fixup(working_file: Path, context: PdfContext):
 
 def optimize_pdf(input_file: Path, context: PdfContext, executor: Executor):
     output_file = context.get_path('optimize.pdf')
-    save_settings = dict(
+    output_pdf, messages = context.plugin_manager.hook.optimize_pdf(
+        input_pdf=input_file,
+        output_pdf=output_file,
+        context=context,
+        executor=executor,
         linearize=should_linearize(input_file, context),
-        **get_pdf_save_settings(context.options.output_type),
     )
-    optimize(input_file, output_file, context, save_settings, executor)
-    return output_file
+
+    input_size = input_file.stat().st_size
+    output_size = output_file.stat().st_size
+    if output_size > 0:
+        ratio = input_size / output_size
+        savings = 1 - output_size / input_size
+        log.info(f"Optimize ratio: {ratio:.2f} savings: {(savings):.1%}")
+
+    return output_pdf, messages
 
 
 def enumerate_compress_ranges(iterable):
@@ -846,11 +865,11 @@ def enumerate_compress_ranges(iterable):
         yield (skipped_from, index), None
 
 
-def merge_sidecars(txt_files: Iterable[Optional[Path]], context: PdfContext):
+def merge_sidecars(txt_files: Iterable[Path | None], context: PdfContext):
     output_file = context.get_path('sidecar.txt')
     with open(output_file, 'w', encoding="utf-8") as stream:
-        for (frm, to), txt_file in enumerate_compress_ranges(txt_files):
-            if frm != 1:
+        for (from_, to_), txt_file in enumerate_compress_ranges(txt_files):
+            if from_ != 1:
                 stream.write('\f')  # Form feed between pages
             if txt_file:
                 with open(txt_file, encoding="utf-8") as in_:
@@ -863,10 +882,10 @@ def merge_sidecars(txt_files: Iterable[Optional[Path]], context: PdfContext):
                     else:
                         stream.write(txt)
             else:
-                if frm != to:
-                    pages = f'{frm}-{to}'
+                if from_ != to_:
+                    pages = f'{from_}-{to_}'
                 else:
-                    pages = f'{frm}'
+                    pages = f'{from_}'
                 stream.write(f'[OCR skipped on page(s) {pages}]')
     return output_file
 
