@@ -1,9 +1,10 @@
-# © 2016 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Implements the concurrent and page synchronous parts of the pipeline."""
+
+
+from __future__ import annotations
 
 import argparse
 import logging
@@ -16,7 +17,7 @@ from concurrent.futures.thread import BrokenThreadPool
 from functools import partial
 from pathlib import Path
 from tempfile import mkdtemp
-from typing import List, NamedTuple, Optional, Tuple, cast
+from typing import NamedTuple, Sequence, cast
 
 import PIL
 
@@ -68,11 +69,13 @@ from ocrmypdf.pdfa import file_claims_pdfa
 log = logging.getLogger(__name__)
 
 
-class PageResult(NamedTuple):  # pylint: disable=inherit-non-class
+class PageResult(NamedTuple):
+    """Result when a page is finished processing."""
+
     pageno: int
-    pdf_page_from_image: Optional[Path]
-    ocr: Optional[Path]
-    text: Optional[Path]
+    pdf_page_from_image: Path | None
+    ocr: Path | None
+    text: Path | None
     orientation_correction: int
 
 
@@ -111,7 +114,7 @@ def preprocess(
 
 def make_intermediate_images(
     page_context: PageContext, orientation_correction: int
-) -> Tuple[Path, Optional[Path]]:
+) -> tuple[Path, Path | None]:
     options = page_context.options
 
     ocr_image = preprocess_out = None
@@ -226,7 +229,9 @@ def exec_page_sync(page_context: PageContext) -> PageResult:
     )
 
 
-def post_process(pdf_file: Path, context: PdfContext, executor: Executor) -> Path:
+def post_process(
+    pdf_file: Path, context: PdfContext, executor: Executor
+) -> tuple[Path, Sequence[str]]:
     pdf_out = pdf_file
     if context.options.output_type.startswith('pdfa'):
         ps_stub_out = generate_postscript_stub(context)
@@ -244,7 +249,7 @@ def worker_init(max_pixels: int) -> None:
     pikepdf_enable_mmap()
 
 
-def exec_concurrent(context: PdfContext, executor: Executor) -> None:
+def exec_concurrent(context: PdfContext, executor: Executor) -> Sequence[str]:
     """Execute the pipeline concurrently"""
 
     # Run exec_page_sync on every page context
@@ -253,7 +258,7 @@ def exec_concurrent(context: PdfContext, executor: Executor) -> None:
     if max_workers > 1:
         log.info("Start processing %d pages concurrently", max_workers)
 
-    sidecars: List[Optional[Path]] = [None] * len(context.pdfinfo)
+    sidecars: list[Path | None] = [None] * len(context.pdfinfo)
     ocrgraft = OcrGrafter(context)
 
     def update_page(result: PageResult, pbar):
@@ -296,13 +301,15 @@ def exec_concurrent(context: PdfContext, executor: Executor) -> None:
     # Merge layers to one single pdf
     pdf = ocrgraft.finalize()
 
+    messages: Sequence[str] = []
     if options.output_type != 'none':
         # PDF/A and metadata
         log.info("Postprocessing...")
-        pdf = post_process(pdf, context, executor)
+        pdf, messages = post_process(pdf, context, executor)
 
         # Copy PDF file to destination
         copy_final(pdf, options.output_file, context)
+    return messages
 
 
 def configure_debug_logging(
@@ -329,7 +336,7 @@ def configure_debug_logging(
 def run_pipeline(
     options: argparse.Namespace,
     *,
-    plugin_manager: Optional[OcrmypdfPluginManager],
+    plugin_manager: OcrmypdfPluginManager | None,
     api: bool = False,
 ) -> ExitCode:
     # Any changes to options will not take effect for options that are already
@@ -382,7 +389,7 @@ def run_pipeline(
         validate_pdfinfo_options(context)
 
         # Execute the pipeline
-        exec_concurrent(context, executor)
+        optimize_messages = exec_concurrent(context, executor)
 
         if options.output_file == '-':
             log.info("Output sent to stdout")
@@ -408,7 +415,9 @@ def run_pipeline(
             if not check_pdf(options.output_file):
                 log.warning('Output file: The generated PDF is INVALID')
                 return ExitCode.invalid_output_pdf
-            report_output_file_size(options, start_input_file, options.output_file)
+            report_output_file_size(
+                options, start_input_file, options.output_file, optimize_messages
+            )
 
     except (KeyboardInterrupt if not api else NeverRaise):
         if options.verbose >= 1:
@@ -425,7 +434,7 @@ def run_pipeline(
         else:
             log.error(type(e).__name__)
         return e.exit_code
-    except (PIL.Image.DecompressionBombError if not api else NeverRaise) as e:
+    except (PIL.Image.DecompressionBombError if not api else NeverRaise):
         log.exception(
             "A decompression bomb error was encountered while executing the "
             "pipeline. Use the argument --max-image-mpixels to raise the maximum "
@@ -435,7 +444,7 @@ def run_pipeline(
     except (
         BrokenProcessPool if not api else NeverRaise,
         BrokenThreadPool if not api else NeverRaise,
-    ) as e:
+    ):
         log.exception(
             "A worker process was terminated unexpectedly. This is known to occur if "
             "processing your file takes all available swap space and RAM. It may "
