@@ -1,31 +1,33 @@
 #!/usr/bin/env python3
-# © 2015-17 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Validate a work order from API or command line."""
+
+from __future__ import annotations
 
 import locale
 import logging
 import os
 import sys
 import unicodedata
+from argparse import Namespace
 from pathlib import Path
 from shutil import copyfileobj
-from typing import List, Set, Tuple
+from typing import Sequence
 
 import pikepdf
 import PIL
+from pluggy import PluginManager
 
-from ocrmypdf._exec import jbig2enc, pngquant, unpaper
+from ocrmypdf._exec import unpaper
 from ocrmypdf.exceptions import (
     BadArgsError,
     InputFileError,
     MissingDependencyError,
     OutputFileAccessError,
 )
-from ocrmypdf.helpers import is_file_writable, monotonic, safe_symlink, samefile
+from ocrmypdf.helpers import is_file_writable, monotonic, safe_symlink
 from ocrmypdf.hocrtransform import HOCR_OK_LANGS
 from ocrmypdf.subprocess import check_external_program
 
@@ -40,8 +42,8 @@ log = logging.getLogger(__name__)
 # --------
 
 
-def check_platform():
-    if os.name == 'nt' and sys.maxsize <= 2 ** 32:  # pragma: no cover
+def check_platform() -> None:
+    if os.name == 'nt' and sys.maxsize <= 2**32:  # pragma: no cover
         # 32-bit interpreter on Windows
         log.error(
             "You are running OCRmyPDF in a 32-bit (x86) Python interpreter."
@@ -49,7 +51,7 @@ def check_platform():
         )
 
 
-def check_options_languages(options, ocr_engine_languages):
+def check_options_languages(options: Namespace, ocr_engine_languages: set[str]) -> None:
     if not options.languages:
         options.languages = {DEFAULT_LANGUAGE}
         system_lang = locale.getlocale()[0]
@@ -68,7 +70,7 @@ def check_options_languages(options, ocr_engine_languages):
         raise MissingDependencyError(msg)
 
 
-def check_options_output(options):
+def check_options_output(options: Namespace) -> None:
     is_latin = options.languages.issubset(HOCR_OK_LANGS)
 
     if options.pdf_renderer.startswith('hocr') and not is_latin:
@@ -104,7 +106,7 @@ def check_options_output(options):
         )
 
 
-def check_options_sidecar(options):
+def check_options_sidecar(options: Namespace) -> None:
     if options.sidecar == '\0':
         if options.output_file == '-':
             raise BadArgsError(
@@ -121,7 +123,7 @@ def check_options_sidecar(options):
         )
 
 
-def check_options_preprocessing(options):
+def check_options_preprocessing(options: Namespace) -> None:
     if options.clean_final:
         options.clean = True
     if options.unpaper_args and not options.clean:
@@ -143,16 +145,16 @@ def check_options_preprocessing(options):
             raise BadArgsError("--unpaper-args: " + str(e)) from e
 
 
-def _pages_from_ranges(ranges: str) -> Set[int]:
-    pages: List[int] = []
+def _pages_from_ranges(ranges: str) -> set[int]:
+    pages: list[int] = []
     page_groups = ranges.replace(' ', '').split(',')
-    for g in page_groups:
-        if not g:
+    for group in page_groups:
+        if not group:
             continue
         try:
-            start, end = g.split('-')
+            start, end = group.split('-')
         except ValueError:
-            pages.append(int(g) - 1)
+            pages.append(int(group) - 1)
         else:
             try:
                 new_pages = list(range(int(start) - 1, int(end)))
@@ -162,7 +164,7 @@ def _pages_from_ranges(ranges: str) -> Set[int]:
                     ) from None
                 pages.extend(new_pages)
             except ValueError:
-                raise BadArgsError(f"invalid page subrange '{g}'") from None
+                raise BadArgsError(f"invalid page subrange '{group}'") from None
 
     if not pages:
         raise BadArgsError(
@@ -182,7 +184,7 @@ def _pages_from_ranges(ranges: str) -> Set[int]:
     return set(pages)
 
 
-def check_options_ocr_behavior(options):
+def check_options_ocr_behavior(options: Namespace) -> None:
     exclusive_options = sum(
         (1 if opt else 0)
         for opt in (options.force_ocr, options.skip_text, options.redo_ocr)
@@ -193,38 +195,7 @@ def check_options_ocr_behavior(options):
         options.pages = _pages_from_ranges(options.pages)
 
 
-def check_options_optimizing(options):
-    if options.optimize >= 2:
-        check_external_program(
-            program='pngquant',
-            package='pngquant',
-            version_checker=pngquant.version,
-            need_version='2.0.1',
-            required_for='--optimize {2,3}',
-        )
-
-    if options.optimize >= 2:
-        # Although we use JBIG2 for optimize=1, don't nag about it unless the
-        # user is asking for more optimization
-        check_external_program(
-            program='jbig2',
-            package='jbig2enc',
-            version_checker=jbig2enc.version,
-            need_version='0.28',
-            required_for='--optimize {2,3} | --jbig2-lossy',
-            recommended=True if not options.jbig2_lossy else False,
-        )
-
-    if options.optimize == 0 and any(
-        [options.jbig2_lossy, options.png_quality, options.jpeg_quality]
-    ):
-        log.warning(
-            "The arguments --jbig2-lossy, --png-quality, and --jpeg-quality "
-            "will be ignored because --optimize=0."
-        )
-
-
-def check_options_advanced(options):
+def check_options_advanced(options: Namespace) -> None:
     if options.pdfa_image_compression != 'auto' and not options.output_type.startswith(
         'pdfa'
     ):
@@ -234,45 +205,48 @@ def check_options_advanced(options):
         )
 
 
-def check_options_metadata(options):
+def check_options_metadata(options: Namespace) -> None:
     docinfo = [options.title, options.author, options.keywords, options.subject]
     for s in (m for m in docinfo if m):
-        for c in s:
-            if unicodedata.category(c) == 'Co' or ord(c) >= 0x10000:
+        for char in s:
+            if unicodedata.category(char) == 'Co' or ord(char) >= 0x10000:
+                hexchar = hex(ord(char))[2:].upper()
                 raise ValueError(
                     "One of the metadata strings contains "
-                    "an unsupported Unicode character: '{}' (U+{})".format(
-                        c, hex(ord(c))[2:].upper()
-                    )
+                    "an unsupported Unicode character: "
+                    f"{char} (U+{hexchar})"
                 )
 
 
-def check_options_pillow(options):
+def check_options_pillow(options: Namespace) -> None:
     PIL.Image.MAX_IMAGE_PIXELS = int(options.max_image_mpixels * 1_000_000)
     if PIL.Image.MAX_IMAGE_PIXELS == 0:
         PIL.Image.MAX_IMAGE_PIXELS = None
 
 
-def _check_options(options, plugin_manager, ocr_engine_languages):
+def _check_plugin_invariant_options(options: Namespace) -> None:
     check_platform()
-    check_options_languages(options, ocr_engine_languages)
     check_options_metadata(options)
     check_options_output(options)
     check_options_sidecar(options)
     check_options_preprocessing(options)
     check_options_ocr_behavior(options)
-    check_options_optimizing(options)
     check_options_advanced(options)
     check_options_pillow(options)
+
+
+def _check_plugin_options(options: Namespace, plugin_manager: PluginManager) -> None:
     plugin_manager.hook.check_options(options=options)
-
-
-def check_options(options, plugin_manager):
     ocr_engine_languages = plugin_manager.hook.get_ocr_engine().languages(options)
-    _check_options(options, plugin_manager, ocr_engine_languages)
+    check_options_languages(options, ocr_engine_languages)
 
 
-def create_input_file(options, work_folder: Path) -> Tuple[Path, str]:
+def check_options(options: Namespace, plugin_manager: PluginManager) -> None:
+    _check_plugin_invariant_options(options)
+    _check_plugin_options(options, plugin_manager)
+
+
+def create_input_file(options: Namespace, work_folder: Path) -> tuple[Path, str]:
     if options.input_file == '-':
         # stdin
         log.info('reading file from standard input')
@@ -293,7 +267,7 @@ def create_input_file(options, work_folder: Path) -> Tuple[Path, str]:
             target = work_folder / 'origin'
             safe_symlink(options.input_file, target)
             return target, os.fspath(options.input_file)
-        except FileNotFoundError:
+        except FileNotFoundError as e:
             msg = f"File not found - {options.input_file}"
             if Path('/.dockerenv').exists():  # pragma: no cover
                 msg += (
@@ -304,10 +278,10 @@ def create_input_file(options, work_folder: Path) -> Tuple[Path, str]:
                     "\n"
                     "\tdocker run -i --rm jbarlow83/ocrmypdf - - <input.pdf >output.pdf\n"
                 )
-            raise InputFileError(msg)
+            raise InputFileError(msg) from e
 
 
-def check_requested_output_file(options):
+def check_requested_output_file(options: Namespace) -> None:
     if options.output_file == '-':
         if sys.stdout.isatty():
             raise BadArgsError(
@@ -324,7 +298,16 @@ def check_requested_output_file(options):
         )
 
 
-def report_output_file_size(options, input_file, output_file):
+def report_output_file_size(
+    options: Namespace,
+    input_file: Path,
+    output_file: Path,
+    optimize_messages: Sequence[str] | None = None,
+    file_overhead: int = 4000,
+    page_overhead: int = 3000,
+) -> None:
+    if optimize_messages is None:
+        optimize_messages = []
     try:
         output_size = Path(output_file).stat().st_size
         input_size = Path(input_file).stat().st_size
@@ -333,9 +316,7 @@ def report_output_file_size(options, input_file, output_file):
     with pikepdf.open(output_file) as p:
         # Overhead constants obtained by estimating amount of data added by OCR
         # PDF/A conversion, and possible XMP metadata addition, with compression
-        FILE_OVERHEAD = 4000
-        OCR_PER_PAGE_OVERHEAD = 3000
-        reasonable_overhead = FILE_OVERHEAD + OCR_PER_PAGE_OVERHEAD * len(p.pages)
+        reasonable_overhead = file_overhead + page_overhead * len(p.pages)
     ratio = output_size / input_size
     reasonable_ratio = output_size / (input_size + reasonable_overhead)
     if reasonable_ratio < 1.35 or input_size < 25000:
@@ -355,19 +336,8 @@ def report_output_file_size(options, input_file, output_file):
                 f"The argument --{arg.replace('_', '-')} was issued, causing transcoding."
             )
 
-    if options.optimize == 0:
-        reasons.append("Optimization was disabled.")
-    else:
-        image_optimizers = {
-            'jbig2': jbig2enc.available(),
-            'pngquant': pngquant.available(),
-        }
-        for name, available in image_optimizers.items():
-            if not available:
-                reasons.append(
-                    f"The optional dependency '{name}' was not found, so some image "
-                    f"optimizations could not be attempted."
-                )
+    reasons.extend(optimize_messages)
+
     if options.output_type.startswith('pdfa'):
         reasons.append("PDF/A conversion was enabled. (Try `--output-type pdf`.)")
     if options.plugins:
