@@ -1,9 +1,9 @@
-# © 2019 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Functions for using ocrmypdf as an API."""
+
+from __future__ import annotations
 
 import logging
 import os
@@ -12,7 +12,7 @@ import threading
 from enum import IntEnum
 from io import IOBase
 from pathlib import Path
-from typing import AnyStr, BinaryIO, Iterable, Optional, Union
+from typing import AnyStr, BinaryIO, Iterable, Union
 from warnings import warn
 
 from ocrmypdf._logging import PageNumberFilter, TqdmConsole
@@ -25,7 +25,10 @@ from ocrmypdf.helpers import is_iterable_notstr
 try:
     import coloredlogs
 except ModuleNotFoundError:
-    coloredlogs = None
+    coloredlogs = None  # pylint: disable=invalid-name
+
+if coloredlogs:
+    from humanfriendly.terminal import enable_ansi_support
 
 
 StrPath = Union[Path, AnyStr]
@@ -37,6 +40,7 @@ _api_lock = threading.Lock()
 class Verbosity(IntEnum):
     """Verbosity level for configure_logging."""
 
+    # pylint: disable=invalid-name
     quiet = -1  #: Suppress most messages
     default = 0  #: Default level of logging
     debug = 1  #: Output ocrmypdf debug messages
@@ -116,16 +120,15 @@ def configure_logging(
         fmt = '%(pageno)s%(message)s'
 
     use_colors = progress_bar_friendly
-    if not coloredlogs:
-        use_colors = False
-    if use_colors:
-        if os.name == 'nt':
-            use_colors = coloredlogs.enable_ansi_support()
+    formatter = None
+    if coloredlogs and use_colors:
+        use_colors = enable_ansi_support()
         if use_colors:
             use_colors = coloredlogs.terminal_supports_colors()
-    if use_colors:
-        formatter = coloredlogs.ColoredFormatter(fmt=fmt)
-    else:
+        if use_colors:
+            formatter = coloredlogs.ColoredFormatter(fmt=fmt)
+
+    if not formatter:
         formatter = logging.Formatter(fmt=fmt)
 
     console.setFormatter(formatter)
@@ -193,7 +196,7 @@ def create_options(
     else:
         cmdline.append(os.fspath(output_file))
 
-    parser._api_mode = True
+    parser.enable_api_mode()
     options = parser.parse_args(cmdline)
     for keyword, val in deferred:
         setattr(options, keyword, val)
@@ -213,7 +216,7 @@ def ocr(  # pylint: disable=unused-argument
     language: Iterable[str] = None,
     image_dpi: int = None,
     output_type=None,
-    sidecar: Optional[StrPath] = None,
+    sidecar: StrPath | None = None,
     jobs: int = None,
     use_threads: bool = None,
     title: str = None,
@@ -295,7 +298,7 @@ def ocr(  # pylint: disable=unused-argument
             text already, and settings did not tell us to proceed.
         ocrmypdf.InputFileError: Any other problem with the input file.
         ocrmypdf.SubprocessOutputError: Any error related to executing a subprocess.
-        ocrmypdf.EncryptedPdfERror: If the input PDF is encrypted (password protected).
+        ocrmypdf.EncryptedPdfError: If the input PDF is encrypted (password protected).
             OCRmyPDF does not remove passwords.
         ocrmypdf.TesseractConfigError: If Tesseract reported its configuration was not
             valid.
