@@ -1,10 +1,10 @@
-# © 2020 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
+"""Built-in plugin to implement OCR using Tesseract."""
 
 
+from __future__ import annotations
+
 import logging
 import os
 
@@ -138,6 +138,8 @@ def validate(pdfinfo, options):
 
 
 class TesseractOcrEngine(OcrEngine):
+    """Implements OCR with Tesseract."""
+
     @staticmethod
     def version():
         return tesseract.version()
