@@ -1,44 +1,29 @@
 #!/usr/bin/env python3
-#
-# Copyright (c) 2010, Jonathan Brinley
-#   Original version from: https://github.com/jbrinley/HocrConverter
-#
-# Copyright (c) 2013-14, Julien Pfefferkorn
-#   Modifications
-#
-# Copyright (c) 2015-16, James R. Barlow
-#   Set text to transparent
-#
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the
-# "Software"), to deal in the Software without restriction, including
-# without limitation the rights to use, copy, modify, merge, publish,
-# distribute, sublicense, and/or sell copies of the Software, and to
-# permit persons to whom the Software is furnished to do so, subject to
-# the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+# SPDX-FileCopyrightText: 2010 Jonathan Brinley
+# SPDX-FileCopyrightText: 2013-2014 Julien Pfefferkorn
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MIT
+
+"""Transform .hocr and page image to text PDF."""
+
+from __future__ import annotations
 
 import argparse
 import os
 import re
+import warnings
 from math import atan, cos, sin
 from pathlib import Path
 from typing import Any, NamedTuple, Optional, Tuple, Union
 from xml.etree import ElementTree
 
-from reportlab.lib.colors import black, cyan, magenta, red
-from reportlab.lib.units import inch
-from reportlab.pdfgen.canvas import Canvas
+with warnings.catch_warnings():
+    warnings.filterwarnings(
+        'ignore', category=DeprecationWarning, message=r".*load_module.*"
+    )
+    from reportlab.lib.colors import black, cyan, magenta, red
+    from reportlab.lib.units import inch
+    from reportlab.pdfgen.canvas import Canvas
 
 # According to Wikipedia these languages are supported in the ISO-8859-1 character
 # set, meaning reportlab can generate them and they are compatible with hocr,
@@ -99,7 +84,7 @@ HOCR_OK_LANGS = frozenset(
 Element = ElementTree.Element
 
 
-class Rect(NamedTuple):  # pylint: disable=inherit-non-class
+class Rect(NamedTuple):
     """A rectangle for managing PDF coordinates."""
 
     x1: Any
@@ -109,7 +94,7 @@ class Rect(NamedTuple):  # pylint: disable=inherit-non-class
 
 
 class HocrTransformError(Exception):
-    pass
+    """Error while applying hOCR transform."""
 
 
 class HocrTransform:
@@ -132,7 +117,7 @@ class HocrTransform:
         {'ﬀ': 'ff', 'ﬃ': 'f‌f‌i', 'ﬄ': 'f‌f‌l', 'ﬁ': 'fi', 'ﬂ': 'fl'}
     )
 
-    def __init__(self, *, hocr_filename: Union[str, Path], dpi: float):
+    def __init__(self, *, hocr_filename: str | Path, dpi: float):
         self.dpi = dpi
         self.hocr = ElementTree.parse(os.fspath(hocr_filename))
 
@@ -196,7 +181,7 @@ class HocrTransform:
         return out
 
     @classmethod
-    def baseline(cls, element: Element) -> Tuple[float, float]:
+    def baseline(cls, element: Element) -> tuple[float, float]:
         """
         Returns a tuple containing the baseline slope and intercept.
         """
@@ -212,7 +197,7 @@ class HocrTransform:
         """
         return Rect._make((c / self.dpi * inch) for c in pxl)
 
-    def _child_xpath(self, html_tag: str, html_class: Optional[str] = None) -> str:
+    def _child_xpath(self, html_tag: str, html_class: str | None = None) -> str:
         xpath = f".//{self.xmlns}{html_tag}"
         if html_class:
             xpath += f"[@class='{html_class}']"
@@ -239,7 +224,7 @@ class HocrTransform:
         self,
         *,
         out_filename: Path,
-        image_filename: Optional[Path] = None,
+        image_filename: Path | None = None,
         show_bounding_boxes: bool = False,
         fontname: str = "Helvetica",
         invisible_text: bool = False,
@@ -287,7 +272,7 @@ class HocrTransform:
                 continue
 
             pxl_coords = self.element_coordinates(elem)
-            pt = self.pt_from_pixel(pxl_coords)
+            pt = self.pt_from_pixel(pxl_coords)  # pylint: disable=invalid-name
 
             # draw the bbox border
             if show_bounding_boxes:  # pragma: no cover
@@ -342,7 +327,7 @@ class HocrTransform:
     def _do_line(
         self,
         pdf: Canvas,
-        line: Optional[Element],
+        line: Element | None,
         elemclass: str,
         fontname: str,
         invisible_text: bool,
