@@ -1,19 +1,19 @@
-# © 2020 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""OCRmyPDF pluggy plugin specification."""
+
+from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from argparse import ArgumentParser, Namespace
 from logging import Handler
 from pathlib import Path
-from typing import TYPE_CHECKING, AbstractSet, List, NamedTuple, Optional
+from typing import TYPE_CHECKING, AbstractSet, NamedTuple, Sequence
 
 import pluggy
 
-from ocrmypdf._concurrent import Executor
+from ocrmypdf import Executor, PdfContext
 from ocrmypdf.helpers import Resolution
 
 if TYPE_CHECKING:
@@ -42,6 +42,33 @@ def get_logging_console() -> Handler:
     """
 
 
+@hookspec
+def initialize(plugin_manager: pluggy.PluginManager):
+    """Called when this plugin is first loaded into OCRmyPDF.
+
+    The primary intended use of this is for plugins to check compatibility with other
+    plugins and possibly block other blocks, a plugin that wishes to block ocrmypdf's
+    built-in optimize plugin could do:
+
+    .. code-block::
+
+        plugin_manager.set_blocked('ocrmypdf.builtin_plugins.optimize')
+
+    It would also be reasonable for an plugin implementation to check if it is unable
+    to proceed, for example, because a required dependency is missing. (If the plugin's
+    ability to proceed depends on options and arguments, use ``validate`` instead.)
+
+    Raises:
+        ocrmypdf.exceptions.ExitCodeException: If options are not acceptable
+            and the application should terminate gracefully with an informative
+            message and error code.
+
+    Note:
+        This hook will be called from the main process, and may modify global state
+        before child worker processes are forked.
+    """
+
+
 @hookspec
 def add_options(parser: ArgumentParser) -> None:
     """Allows the plugin to add its own command line and API arguments.
@@ -132,6 +159,7 @@ def get_progressbar_class():
     Here is how OCRmyPDF will use the progress bar:
 
     Example:
+
         pbar_class = pm.hook.get_progressbar_class()
         with pbar_class(**tqdm_kwargs) as pbar:
             ...
@@ -140,7 +168,7 @@ def get_progressbar_class():
 
 
 @hookspec
-def validate(pdfinfo: 'PdfInfo', options: Namespace) -> None:
+def validate(pdfinfo: PdfInfo, options: Namespace) -> None:
     """Called to give a plugin an opportunity to review *options* and *pdfinfo*.
 
     *options* contains the "work order" to process a particular file. *pdfinfo*
@@ -166,8 +194,8 @@ def rasterize_pdf_page(
     raster_device: str,
     raster_dpi: Resolution,
     pageno: int,
-    page_dpi: Optional[Resolution],
-    rotation: Optional[int],
+    page_dpi: Resolution | None,
+    rotation: int | None,
     filter_vector: bool,
 ) -> Path:
     """Rasterize one page of a PDF at resolution raster_dpi in canvas units.
@@ -196,7 +224,7 @@ def rasterize_pdf_page(
 
 
 @hookspec(firstresult=True)
-def filter_ocr_image(page: 'PageContext', image: 'Image.Image') -> 'Image.Image':
+def filter_ocr_image(page: PageContext, image: Image.Image) -> Image.Image:
     """Called to filter the image before it is sent to OCR.
 
     This is the image that OCR sees, not what the user sees when they view the
@@ -223,7 +251,7 @@ def filter_ocr_image(page: 'PageContext', image: 'Image.Image') -> 'Image.Image'
 
 
 @hookspec(firstresult=True)
-def filter_page_image(page: 'PageContext', image_filename: Path) -> Path:
+def filter_page_image(page: PageContext, image_filename: Path) -> Path:
     """Called to filter the whole page before it is inserted into the PDF.
 
     A whole page image is only produced when preprocessing command line arguments
@@ -235,9 +263,9 @@ def filter_page_image(page: 'PageContext', image_filename: Path) -> Path:
     ``image_filename``. The hook may overwrite ``image_filename`` with a new file.
 
     The output image should preserve the same physical unit dimensions, that is
-    (width * dpi_x, height * dpi_y). That is, if the image is resized, the DPI
+    ``(width * dpi_x, height * dpi_y)``. That is, if the image is resized, the DPI
     must be adjusted by the reciprocal. If this is not preserved, the PDF page
-    will be resized and the OCR layer misaligned. OCRmyPDF does not nothing
+    will be resized and the OCR layer misaligned. OCRmyPDF does nothing
     to enforce these constraints; it is up to the plugin to do sensible things.
 
     OCRmyPDF will create the PDF page based on the image format used (unless the
@@ -263,9 +291,7 @@ def filter_page_image(page: 'PageContext', image_filename: Path) -> Path:
 
 
 @hookspec(firstresult=True)
-def filter_pdf_page(
-    page: 'PageContext', image_filename: Path, output_pdf: Path
-) -> Path:
+def filter_pdf_page(page: PageContext, image_filename: Path, output_pdf: Path) -> Path:
     """Called to convert a filtered whole page image into a PDF.
 
     A whole page image is only produced when preprocessing command line arguments
@@ -399,8 +425,7 @@ def get_ocr_engine() -> OcrEngine:
     """Returns an OcrEngine to use for processing this file.
 
     The OcrEngine may be instantiated multiple times, by both the main process
-    and child process. As such, it must be obtain store any state in ``options``
-    or some common location.
+    and child process.
 
     Note:
         This is a :ref:`firstresult hook<firstresult>`.
@@ -409,7 +434,7 @@ def get_ocr_engine() -> OcrEngine:
 
 @hookspec(firstresult=True)
 def generate_pdfa(
-    pdf_pages: List[Path],
+    pdf_pages: list[Path],
     pdfmark: Path,
     output_file: Path,
     compression: str,
@@ -456,3 +481,63 @@ def generate_pdfa(
     See also:
         https://github.com/tqdm/tqdm
     """
+
+
+@hookspec(firstresult=True)
+def optimize_pdf(
+    input_pdf: Path,
+    output_pdf: Path,
+    context: PdfContext,
+    executor: Executor,
+    linearize: bool,
+) -> tuple[Path, Sequence[str]]:
+    """Optimize a PDF after image, OCR and metadata processing.
+
+    If the input_pdf is a PDF/A, the plugin should modify input_pdf in a way
+    that preserves the PDF/A status, or report to the user when this is not possible.
+
+    If the implementation fails to produce a smaller file than the input file, it
+    should return input_pdf instead.
+
+    A plugin that implements a new optimizer may need to suppress the built-in
+    optimizer by implementing an ``initialize`` hook.
+
+    Arguments:
+        input_pdf: The input PDF, which has OCR added.
+        output_pdf: The requested filename of the output PDF which should be created
+            by this optimization hook.
+        context: The current context.
+        executor: An initialized executor which may be used during optimization,
+            to distribute optimization tasks.
+        linearize: If True, OCRmyPDF requires ``optimize_pdf`` to return a linearized,
+            also known as fast web view PDF.
+
+    Returns:
+        Path: If optimization is successful, the hook should return ``output_file``.
+            If optimization does not produce a smaller file, the hook should return
+            ``input_file``.
+        Sequence[str]: Any comments that the plugin wishes to report to the user,
+            especially reasons it was not able to further optimize the file. For
+            example, the plugin could report that a required third party was not
+            installed, so a specific optimization was not attempted.
+
+    Note:
+        This is a :ref:`firstresult hook<firstresult>`.
+    """
+
+
+@hookspec(firstresult=True)
+def is_optimization_enabled(context: PdfContext) -> bool:
+    """For a given PdfContext, OCRmyPDF asks the plugin if optimization is enabled.
+
+    An optimization plugin might be installed and active but could be disabled by
+    user settings.
+
+    If this returns False, OCRmyPDF will take certain actions to finalize the PDF.
+
+    Returns:
+        True if the plugin's optimization is enabled.
+
+    Note:
+        This is a :ref:`firstresult hook<firstresult>`.
+    """
