@@ -1,9 +1,7 @@
-# © 2017 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 import os
 import platform
@@ -52,7 +50,7 @@ def resources() -> Path:
 
 
 @pytest.fixture
-def ocrmypdf_exec() -> List[str]:
+def ocrmypdf_exec() -> list[str]:
     return [sys.executable, '-m', 'ocrmypdf']
 
 
@@ -134,8 +132,7 @@ def run_ocrmypdf(
 
     p = run(
         p_args,
-        stdout=PIPE,
-        stderr=PIPE,
+        capture_output=True,
         text=text,
         check=False,
     )
