@@ -1,9 +1,7 @@
-# © 2015-19 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 import os
 import shutil
@@ -465,12 +463,18 @@ def test_overlay(resources, outpdf):
     )
 
 
-def test_destination_not_writable(resources, outdir):
-    if os.name != 'nt' and (os.getuid() == 0 or os.geteuid() == 0):
-        pytest.xfail(reason="root can write to anything")
+@pytest.fixture
+def protected_file(outdir):
     protected_file = outdir / 'protected.pdf'
     protected_file.touch()
     protected_file.chmod(0o400)  # Read-only
+    yield protected_file
+
+
+@pytest.mark.skipif(
+    os.name == 'nt' or os.geteuid() == 0, reason="root can write to anything"
+)
+def test_destination_not_writable(resources, protected_file):
     p = run_ocrmypdf(
         resources / 'jbig2.pdf',
         protected_file,
@@ -480,7 +484,8 @@ def test_destination_not_writable(resources, outdir):
     assert p.returncode == ExitCode.file_access_error, "Expected error"
 
 
-def test_tesseract_config_valid(resources, outdir):
+@pytest.fixture
+def valid_tess_config(outdir):
     cfg_file = outdir / 'test.cfg'
     with cfg_file.open('w') as f:
         f.write(
@@ -490,20 +495,22 @@ language_model_penalty_non_dict_word 0
 language_model_penalty_non_freq_dict_word 0
 '''
         )
+    yield cfg_file
 
+
+def test_tesseract_config_valid(resources, valid_tess_config, outpdf):
     check_ocrmypdf(
         resources / '3small.pdf',
-        outdir / 'out.pdf',
+        outpdf,
         '--tesseract-config',
-        cfg_file,
+        valid_tess_config,
         '--pages',
         '1',
     )
 
 
-@pytest.mark.slow  # This test sometimes times out in CI
-@pytest.mark.parametrize('renderer', RENDERERS)
-def test_tesseract_config_invalid(renderer, resources, outdir):
+@pytest.fixture
+def invalid_tess_config(outdir):
     cfg_file = outdir / 'test.cfg'
     with cfg_file.open('w') as f:
         f.write(
@@ -511,14 +518,19 @@ def test_tesseract_config_invalid(renderer, resources, outdir):
 THIS FILE IS INVALID
 '''
         )
+    yield cfg_file
 
+
+@pytest.mark.slow  # This test sometimes times out in CI
+@pytest.mark.parametrize('renderer', RENDERERS)
+def test_tesseract_config_invalid(renderer, resources, invalid_tess_config, outpdf):
     p = run_ocrmypdf(
         resources / 'ccitt.pdf',
-        outdir / 'out.pdf',
+        outpdf,
         '--pdf-renderer',
         renderer,
         '--tesseract-config',
-        cfg_file,
+        invalid_tess_config,
     )
     assert (
         "parameter not found" in p.stderr.lower()
@@ -630,8 +642,7 @@ def test_compression_preserved(ocrmypdf_exec, resources, image, outpdf):
         ]
         p = run(
             p_args,
-            stdout=PIPE,
-            stderr=PIPE,
+            capture_output=True,
             stdin=input_stream,
             text=True,
             check=False,
@@ -691,8 +702,7 @@ def test_compression_changed(ocrmypdf_exec, resources, image, compression, outpd
         ]
         p = run(
             p_args,
-            stdout=PIPE,
-            stderr=PIPE,
+            capture_output=True,
             stdin=input_stream,
             text=True,
             check=False,
@@ -803,6 +813,9 @@ def test_text_curves(resources, outpdf):
         info = PdfInfo(outpdf)
         assert len(info.pages[0].images) == 0, "added images to the vector PDF"
 
+
+def test_text_curves_force(resources, outpdf):
+    with patch('ocrmypdf._pipeline.VECTOR_PAGE_DPI', 100):
         check_ocrmypdf(
             resources / 'vector.pdf',
             outpdf,
@@ -924,3 +937,25 @@ def test_outputtype_none(resources, outtxt):
         'tests/plugins/tesseract_noop.py',
     )
     assert p.returncode == ExitCode.ok
+
+
+@pytest.fixture
+def graph_bad_icc(resources, outdir):
+    synth_input_file = outdir / 'graph-bad-icc.pdf'
+    with pikepdf.open(resources / 'graph.pdf') as pdf:
+        icc = pdf.make_stream(
+            b'invalid icc profile', N=3, Alternate=pikepdf.Name.DeviceRGB
+        )
+        pdf.pages[0].Resources.XObject['/Im0'].ColorSpace = pikepdf.Array(
+            [pikepdf.Name.ICCBased, icc]
+        )
+        pdf.save(synth_input_file)
+        yield synth_input_file
+
+
+def test_corrupt_icc(graph_bad_icc, outpdf, caplog):
+    result = run_ocrmypdf_api(graph_bad_icc, outpdf)
+    assert result == ExitCode.ok
+    assert any(
+        'corrupt or unreadable ICC profile' in rec.message for rec in caplog.records
+    )
