@@ -1,15 +1,12 @@
-# © 2018 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 import datetime
+import warnings
 from datetime import timezone
-from os import fspath
 from shutil import copyfile
-from unittest.mock import patch
 
 import pikepdf
 import pytest
@@ -17,7 +14,7 @@ from pikepdf.models.metadata import decode_pdf_date
 
 from ocrmypdf._jobcontext import PdfContext
 from ocrmypdf._pipeline import convert_to_pdfa, metadata_fixup
-from ocrmypdf._plugin_manager import get_plugin_manager
+from ocrmypdf._plugin_manager import get_parser_options_plugins, get_plugin_manager
 from ocrmypdf.cli import get_parser
 from ocrmypdf.exceptions import ExitCode
 from ocrmypdf.pdfa import file_claims_pdfa, generate_pdfa_ps
@@ -173,6 +170,19 @@ def test_creation_date_preserved(output_type, resources, infile, outpdf):
     assert seconds_between_dates(date_after, datetime.datetime.now(timezone.utc)) < 1000
 
 
+@pytest.fixture
+def libxmp_file_to_dict():
+    try:
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", DeprecationWarning)
+            from libxmp.utils import (
+                file_to_dict,  # pylint: disable=import-outside-toplevel
+            )
+    except Exception:  # pylint: disable=broad-except
+        pytest.skip("libxmp not available or libexempi3 not installed")
+    return file_to_dict
+
+
 @pytest.mark.parametrize(
     'test_file,output_type',
     [
@@ -182,15 +192,12 @@ def test_creation_date_preserved(output_type, resources, infile, outpdf):
         ('3small.pdf', 'pdfa'),
     ],
 )
-def test_xml_metadata_preserved(test_file, output_type, resources, outpdf):
+def test_xml_metadata_preserved(
+    libxmp_file_to_dict, test_file, output_type, resources, outpdf
+):
     input_file = resources / test_file
 
-    try:
-        from libxmp.utils import file_to_dict  # pylint: disable=import-outside-toplevel
-    except Exception:  # pylint: disable=broad-except
-        pytest.skip("libxmp not available or libexempi3 not installed")
-
-    before = file_to_dict(str(input_file))
+    before = libxmp_file_to_dict(str(input_file))
 
     check_ocrmypdf(
         input_file,
@@ -202,7 +209,7 @@ def test_xml_metadata_preserved(test_file, output_type, resources, outpdf):
         'tests/plugins/tesseract_noop.py',
     )
 
-    after = file_to_dict(str(outpdf))
+    after = libxmp_file_to_dict(str(outpdf))
 
     equal_properties = [
         'dc:contributor',
@@ -290,8 +297,8 @@ def test_kodak_toc(resources, outpdf):
 
 
 def test_metadata_fixup_warning(resources, outdir, caplog):
-    options = get_parser().parse_args(
-        args=['--output-type', 'pdfa-2', 'graph.pdf', 'out.pdf']
+    _parser, options, _pm = get_parser_options_plugins(
+        ['--output-type', 'pdfa-2', 'graph.pdf', 'out.pdf']
     )
 
     copyfile(resources / 'graph.pdf', outdir / 'graph.pdf')
@@ -316,6 +323,9 @@ def test_metadata_fixup_warning(resources, outdir, caplog):
     assert any(record.levelname == 'WARNING' for record in caplog.records)
 
 
+XMP_MAGIC = b'W5M0MpCehiHzreSzNTczkc9d'
+
+
 def test_prevent_gs_invalid_xml(resources, outdir):
     generate_pdfa_ps(outdir / 'pdfa.ps')
     copyfile(resources / 'trivial.pdf', outdir / 'layers.rendered.pdf')
@@ -342,7 +352,7 @@ def test_prevent_gs_invalid_xml(resources, outdir):
     contents = (outdir / 'pdfa.pdf').read_bytes()
     # Since the XML may be invalid, we scan instead of actually feeding it
     # to a parser.
-    XMP_MAGIC = b'W5M0MpCehiHzreSzNTczkc9d'
+
     xmp_start = contents.find(XMP_MAGIC)
     xmp_end = contents.rfind(b'<?xpacket end', xmp_start)
     assert 0 < xmp_start < xmp_end
