@@ -1,9 +1,7 @@
-# © 2019 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 import os
 import sys
@@ -30,12 +28,12 @@ def test_stdin(ocrmypdf_exec, resources, outpdf):
             '--plugin',
             'tests/plugins/tesseract_noop.py',
         ]
-        run(p_args, stdout=PIPE, stderr=PIPE, stdin=input_stream, check=True)
+        run(p_args, capture_output=True, stdin=input_stream, check=True)
 
 
 def test_stdout(ocrmypdf_exec, resources, outpdf):
     if 'COV_CORE_DATAFILE' in os.environ:
-        pytest.skip(msg="Coverage uses stdout")
+        pytest.skip("Coverage uses stdout")
 
     input_file = str(resources / 'francais.pdf')
     output_file = str(outpdf)
@@ -53,26 +51,13 @@ def test_stdout(ocrmypdf_exec, resources, outpdf):
     assert check_pdf(output_file)
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 7, 0), reason='better utf-8')
-@pytest.mark.skipif(
-    Path('/etc/alpine-release').exists(), reason="invalid test on alpine"
-)
-@pytest.mark.skipif(os.name == 'nt', reason="invalid test on Windows")
-def test_bad_locale(monkeypatch):
-    monkeypatch.setenv('LC_ALL', 'C')
-    p = run_ocrmypdf('a', 'b')
-    assert p.stdout == '', "stdout not clean"
-    assert p.returncode != 0
-    assert 'configured to use ASCII as encoding' in p.stderr, "should whine"
-
-
 @pytest.mark.xfail(
     os.name == 'nt' and sys.version_info < (3, 8),
     reason="Windows does not like this; not sure how to fix",
 )
 def test_dev_null(resources):
     if 'COV_CORE_DATAFILE' in os.environ:
-        pytest.skip(msg="Coverage uses stdout")
+        pytest.skip("Coverage uses stdout")
 
     p = run_ocrmypdf(
         resources / 'trivial.pdf',
