@@ -1,9 +1,7 @@
-# © 2019 James R. Barlow: github.com/jbarlow83
-#
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+from __future__ import annotations
 
 import logging
 import os
@@ -44,29 +42,26 @@ def make_opts(*args, **kwargs):
 
 
 def test_hocr_notlatin_warning(caplog):
-    # Bypass the test to see if the language is installed; we just want to pretend
-    # that a non-Latin language is installed
-    vd._check_options(
-        *make_opts_pm(language='chi_sim', pdf_renderer='hocr', output_type='pdfa'),
-        {'chi_sim'},
+    vd.check_options_output(
+        make_opts_pm(language='chi_sim', pdf_renderer='hocr', output_type='pdfa')[0]
     )
     assert 'PDF renderer is known to cause' in caplog.text
 
 
 def test_old_ghostscript(caplog):
-    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.19'):
-        vd._check_options(
-            *make_opts_pm(language='chi_sim', output_type='pdfa'), {'chi_sim'}
-        )
+    with patch('ocrmypdf._exec.ghostscript.version', return_value='9.19'), patch(
+        'ocrmypdf._exec.tesseract.get_languages', return_value={'eng', 'chi_sim'}
+    ):
+        vd.check_options(*make_opts_pm(language='chi_sim', output_type='pdfa'))
         assert 'does not work correctly' in caplog.text
 
     with patch('ocrmypdf._exec.ghostscript.version', return_value='9.18'):
         with pytest.raises(MissingDependencyError):
-            vd._check_options(*make_opts_pm(output_type='pdfa-3'), set())
+            vd.check_options(*make_opts_pm(output_type='pdfa-3'))
 
     with patch('ocrmypdf._exec.ghostscript.version', return_value='9.24'):
         with pytest.raises(MissingDependencyError):
-            vd._check_options(*make_opts_pm(), set())
+            vd.check_options(*make_opts_pm())
 
 
 def test_old_tesseract_error():
@@ -74,7 +69,7 @@ def test_old_tesseract_error():
         with pytest.raises(MissingDependencyError):
             opts = make_opts(pdf_renderer='sandwich', language='eng')
             plugin_manager = get_plugin_manager(opts.plugins)
-            vd._check_options(opts, plugin_manager, {'eng'})
+            vd.check_options(opts, plugin_manager)
 
 
 def test_lossless_redo():
@@ -92,9 +87,9 @@ def test_mutex_options():
 
 
 def test_optimizing(caplog):
-    vd.check_options_optimizing(
-        make_opts(optimize=0, jbig2_lossy=True, png_quality=18, jpeg_quality=10)
-    )
+    opts = make_opts(optimize=0, jbig2_lossy=True, png_quality=18, jpeg_quality=10)
+    plugin_manager = get_plugin_manager(opts.plugins)
+    vd.check_options(opts, plugin_manager)
     assert 'will be ignored because' in caplog.text
 
 
@@ -102,7 +97,7 @@ def test_user_words(caplog):
     with patch('ocrmypdf._exec.tesseract.has_user_words', return_value=False):
         opts = make_opts(user_words='foo')
         plugin_manager = get_plugin_manager(opts.plugins)
-        vd._check_options(opts, plugin_manager, set())
+        vd.check_options(opts, plugin_manager)
         assert (
             'Tesseract 4.0 (which you have installed) ignores --user-words'
             in caplog.text
@@ -111,7 +106,7 @@ def test_user_words(caplog):
     with patch('ocrmypdf._exec.tesseract.has_user_words', return_value=True):
         opts = make_opts(user_patterns='foo')
         plugin_manager = get_plugin_manager(opts.plugins)
-        vd._check_options(opts, plugin_manager, set())
+        vd.check_options(opts, plugin_manager)
         assert (
             'Tesseract 4.0 (which you have installed) ignores --user-words'
             not in caplog.text
@@ -145,22 +140,16 @@ def test_report_file_size(tmp_path, caplog):
     pdf.Root.Dummy2 = waste_of_space + waste_of_space
     pdf.save(out)
 
-    with patch('ocrmypdf._validation.jbig2enc.available', return_value=True), patch(
-        'ocrmypdf._validation.pngquant.available', return_value=True
-    ):
-        vd.report_output_file_size(opts, in_, out)
-        assert 'No reason' in caplog.text
+    vd.report_output_file_size(opts, in_, out, ['The optional dependency...'])
+    assert 'optional dependency' in caplog.text
     caplog.clear()
 
-    with patch('ocrmypdf._validation.jbig2enc.available', return_value=False), patch(
-        'ocrmypdf._validation.pngquant.available', return_value=True
-    ):
-        vd.report_output_file_size(opts, in_, out)
-        assert 'optional dependency' in caplog.text
+    vd.report_output_file_size(opts, in_, out, [])
+    assert 'No reason' in caplog.text
     caplog.clear()
 
     opts = make_opts(in_, out, optimize=0, output_type='pdf')
-    vd.report_output_file_size(opts, in_, out)
+    vd.report_output_file_size(opts, in_, out, ["Optimization was disabled."])
     assert 'disabled' in caplog.text
     caplog.clear()
 
@@ -177,7 +166,7 @@ def test_no_progress_bar(progress_bar, resources):
     opts = make_opts(progress_bar=progress_bar, input_file=(resources / 'trivial.pdf'))
     plugin_manager = get_plugin_manager(opts.plugins)
 
-    vd._check_options(opts, plugin_manager, set())
+    vd.check_options(opts, plugin_manager)
 
     pbar_disabled = None
 
@@ -298,13 +287,19 @@ def test_optional_program_recommended(caplog):
 def test_pagesegmode_warning(caplog):
     opts = make_opts(tesseract_pagesegmode='0')
     plugin_manager = get_plugin_manager(opts.plugins)
-    vd._check_options(opts, plugin_manager, set())
+    vd.check_options(opts, plugin_manager)
     assert 'disable OCR' in caplog.text
 
 
 def test_two_languages():
-    vd._check_options(
-        *make_opts_pm(language='fakelang1+fakelang2'), {'fakelang1', 'fakelang2'}
+    vd.check_options_languages(
+        create_options(
+            input_file='a.pdf',
+            output_file='b.pdf',
+            parser=get_parser(),
+            language='fakelang1+fakelang2',
+        ),
+        {'fakelang1', 'fakelang2'},
     )
 
 
